@@ -12,6 +12,8 @@
 #include <netdb.h>
 #include <sys/socket.h>
 #include <sys/wait.h>
+#include <math.h>
+#include <errno.h>
 
 #include <string>
 #include <vector>
@@ -63,9 +65,31 @@ namespace {
     {
       char buf[256];
 
-      if (gethostname(buf, sizeof buf) == 0)
-	if (struct hostent *h = gethostbyname(buf))
-	  this->hostname = h->h_name;
+      if (gethostname(buf, sizeof buf) == -1) {
+	g_warning("gethostname failed : %s", strerror(errno));
+	return;
+      }
+
+      struct hostent *h;
+
+      if (not (h = gethostbyname(buf))) {
+	g_warning("gethostbyname failed : %s", strerror(errno));
+	return;
+      }
+
+      this->hostname = h->h_name;
+
+      if (this->hostname.find("localhost") == 0) {
+	for (char **p = h->h_aliases; *p != NULL; ++p) {
+	  const string alias(*p);
+	  if (alias.find("localhost") == 0)
+	    continue;
+
+	  if (this->hostname.find("localhost") == 0
+	      or alias.size() > this->hostname.size())
+	    this->hostname = alias;
+	}
+      }
     }
 
     void load_memory_info()
@@ -108,6 +132,13 @@ namespace {
       this->free_space_bytes = 0;
 
       for (guint i = 0; i != mountlist.number; ++i) {
+
+	if (string(entries[i].devname).find("/dev/") != 0)
+	  continue;
+
+	if (string(entries[i].mountdir).find("/media/") == 0)
+	  continue;
+
 	glibtop_fsusage usage;
 	glibtop_get_fsusage(&usage, entries[i].mountdir);
 	this->free_space_bytes += usage.bavail * usage.block_size;
@@ -117,6 +148,28 @@ namespace {
     }
   };
 
+  class SolarisSysInfo
+    : public SysInfo
+  {
+  public:
+    SolarisSysInfo()
+      : SysInfo("Solaris")
+    {
+      this->load_solaris_info();
+    }
+
+  private:
+    void load_solaris_info()
+    {
+      std::ifstream input("/etc/release");
+
+      if (input)
+	std::getline(input, this->distro_release);
+      else
+	this->distro_release = _("Unknown version");
+    }
+  };
+
 
   class LSBSysInfo
     : public SysInfo
@@ -197,12 +250,62 @@ namespace {
       g_free(p);
       return new LSBSysInfo;
     }
+    else if (g_file_test("/etc/release", G_FILE_TEST_EXISTS))
+      return new SolarisSysInfo;
 
     return new SysInfo(_("Unknown distribution"));
   }
 }
 
 
+#define X_PAD  5
+#define Y_PAD  12
+#define LOGO_W 92
+#define LOGO_H 351
+#define RADIUS 5
+
+static gboolean
+sysinfo_logo_expose (GtkWidget *widget,
+		     GdkEventExpose *event,
+		     gpointer data_ptr)
+{
+  cairo_t *cr;
+  cairo_pattern_t *cp;
+
+  cr = gdk_cairo_create(widget->window);
+
+  cairo_translate(cr, event->area.x, event->area.y);
+
+  cairo_move_to(cr, X_PAD + RADIUS, Y_PAD);
+  cairo_line_to(cr, X_PAD + LOGO_W - RADIUS, Y_PAD);
+  cairo_arc(cr, X_PAD + LOGO_W - RADIUS, Y_PAD + RADIUS, RADIUS, -0.5 * M_PI, 0);
+  cairo_line_to(cr, X_PAD + LOGO_W, Y_PAD + LOGO_H - RADIUS);
+  cairo_arc(cr, X_PAD + LOGO_W - RADIUS, Y_PAD + LOGO_H - RADIUS, RADIUS, 0, 0.5 * M_PI);
+  cairo_line_to(cr, X_PAD + RADIUS, Y_PAD + LOGO_H);
+  cairo_arc(cr, X_PAD + RADIUS, Y_PAD + LOGO_H - RADIUS, RADIUS, 0.5 * M_PI, -1.0 * M_PI);
+  cairo_line_to(cr, X_PAD, Y_PAD + RADIUS);
+  cairo_arc(cr,  X_PAD + RADIUS, Y_PAD + RADIUS, RADIUS, -1.0 * M_PI, -0.5 * M_PI);
+
+  cp = cairo_pattern_create_linear(0, Y_PAD, 0, Y_PAD + LOGO_H);
+  cairo_pattern_add_color_stop_rgba(cp, 0.0,
+				    widget->style->base[GTK_STATE_SELECTED].red / 65535.0,
+				    widget->style->base[GTK_STATE_SELECTED].green / 65535.0,
+				    widget->style->base[GTK_STATE_SELECTED].blue / 65535.0,
+				    1.0);
+  cairo_pattern_add_color_stop_rgba(cp, 1.0,
+				    widget->style->base[GTK_STATE_SELECTED].red / 65535.0,
+				    widget->style->base[GTK_STATE_SELECTED].green / 65535.0,
+				    widget->style->base[GTK_STATE_SELECTED].blue / 65535.0,
+				    0.0);
+  cairo_set_source(cr, cp);
+  cairo_fill(cr);
+
+  cairo_pattern_destroy(cp);
+  cairo_destroy(cr);
+
+  return FALSE;
+}
+
 GtkWidget *
 procman_create_sysinfo_view(void)
 {
@@ -211,7 +314,7 @@ procman_create_sysinfo_view(void)
 
   SysInfo *data = get_sysinfo();;
 
-  GtkWidget *logo;
+  GtkWidget * logo;
 
   GtkWidget *distro_frame;
   GtkWidget *distro_release_label;
@@ -241,6 +344,9 @@ procman_create_sysinfo_view(void)
   gtk_misc_set_padding(GTK_MISC(logo), 5, 12);
   gtk_box_pack_start(GTK_BOX(hbox), logo, FALSE, FALSE, 0);
 
+  g_signal_connect(G_OBJECT(logo), "expose-event",
+		   G_CALLBACK(sysinfo_logo_expose), NULL);
+
   vbox = gtk_vbox_new(FALSE, 12);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
   gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
@@ -382,7 +488,7 @@ procman_create_sysinfo_view(void)
   gtk_container_set_border_width(GTK_CONTAINER(disk_space_table), 6);
   gtk_container_add(GTK_CONTAINER(alignment), disk_space_table);
 
-  header = gtk_label_new(_("Available Disk Space:"));
+  header = gtk_label_new(_("Available disk space:"));
   gtk_misc_set_alignment(GTK_MISC(header), 0.0, 0.5);
   gtk_table_attach(
 		   GTK_TABLE(disk_space_table), header,
